using System;

namespace ActionExamples
{
    public class ActionsWithParameters
    {
        public static void Runner()
        {
            // Printer(..) takes an Action with parameters 

            // I am deliberately choosing different names for the parameters passed into the Actions, someText, someMessage and text, 
            // none of which match parameter to the method Printer(..) 

            Printer((text) => PrinterMethod(text), "Calling print method");
            Printer(PrinterMethod, "Hi there");

            Printer(someMessage => Console.WriteLine($"Hello from Console.WriteLine - {someMessage}"), "this is a lambda");

            Action<string> action = (s => Console.WriteLine(s));
            Printer(action, "hello");

            Console.WriteLine("Running the actions with with parameters");

            Action<string> printerActionWithParametersAsLambda = (someText) => Console.WriteLine($"Explicitly defined printer action - {someText}");
            Printer(printerActionWithParametersAsLambda, "printerActionAsLambda");

            Action<string> printerActionWithParametersAsMethod = PrinterMethod;
            Printer(printerActionWithParametersAsMethod, "printerActionAsMethod");

            // note that you don't need () when a single parameter is passed, but it is common to include them
            Printer(someMessage => Console.WriteLine($"Hello from Console.WriteLine - {someMessage}"), "this is a lambda");
            

            Console.WriteLine();
        }

        public static void Printer(Action<string> myPrinterAction, string value)
        {
            myPrinterAction(value);
        }

        public static void PrinterMethod(string text)
        {
            Console.WriteLine($"Hello from PrinterMethod - {text}");
        }
    }
}